@@ -5,7 +5,7 @@ use core::{marker::PhantomData, time::Duration};
 use serde::{de::DeserializeOwned, Serialize};
 
 #[cfg(feature = "std")]
-use core::ptr::read_volatile;
+use core::ptr::{addr_of, read_volatile};
 
 #[cfg(feature = "std")]
 use crate::bolts::{
@@ -15,7 +15,7 @@ use crate::bolts::{
 
 use crate::{
     bolts::{
-        llmp::{self, Flag, LlmpClientDescription, LlmpSender, Tag},
+        llmp::{self, Flags, LlmpClientDescription, LlmpSender, Tag},
         shmem::ShMemProvider,
     },
     corpus::CorpusScheduler,
@@ -54,6 +54,8 @@ const LLMP_TAG_EVENT_TO_BOTH: llmp::Tag = 0x2B0741;
 const _LLMP_TAG_RESTART: llmp::Tag = 0x8357A87;
 const _LLMP_TAG_NO_RESTART: llmp::Tag = 0x57A7EE71;
 
+/// An [`EventManager`] that forwards all events to other attached fuzzers on shared maps or via tcp,
+/// using low-level message passing, [`crate::bolts::llmp`].
 #[derive(Debug)]
 pub struct LlmpEventManager<I, S, SP, ST>
 where
@@ -109,7 +111,7 @@ where
         })
     }
 
-    /// If a client respawns, it may reuse the existing connection, previously stored by LlmpClient::to_env
+    /// If a client respawns, it may reuse the existing connection, previously stored by [`LlmpClient::to_env()`].
     #[cfg(feature = "std")]
     pub fn existing_client_from_env(shmem_provider: SP, env_name: &str) -> Result<Self, Error> {
         Ok(Self {
@@ -149,7 +151,7 @@ where
         })
     }
 
-    /// Write the config for a client eventmgr to env vars, a new client can reattach using existing_client_from_env
+    /// Write the config for a client [`EventManager`] to env vars, a new client can reattach using [`LlmpEventManager::existing_client_from_env()`].
     #[cfg(feature = "std")]
     pub fn to_env(&self, env_name: &str) {
         match &self.llmp {
@@ -173,7 +175,7 @@ where
                 #[cfg(feature = "llmp_compression")]
                 let compressor = &self.compressor;
                 broker.loop_forever(
-                    &mut |sender_id: u32, tag: Tag, _flags: Flag, msg: &[u8]| {
+                    &mut |sender_id: u32, tag: Tag, _flags: Flags, msg: &[u8]| {
                         if tag == LLMP_TAG_EVENT_TO_BOTH {
                             #[cfg(not(feature = "llmp_compression"))]
                             let event_bytes = msg;
@@ -292,11 +294,10 @@ where
 
                 let observers: OT = postcard::from_bytes(&observers_buf)?;
                 // TODO include ExitKind in NewTestcase
-                let fitness = state.is_interesting(&input, &observers, &ExitKind::Ok)?;
-                if fitness > 0
-                    && state
-                        .add_if_interesting(&input, fitness, scheduler)?
-                        .is_some()
+                let is_interesting = state.is_interesting(&input, &observers, &ExitKind::Ok)?;
+                if state
+                    .add_if_interesting(&input, is_interesting, scheduler)?
+                    .is_some()
                 {
                     #[cfg(feature = "std")]
                     println!("Added received Testcase");
@@ -376,7 +377,7 @@ where
     #[cfg(feature = "llmp_compression")]
     fn fire(&mut self, _state: &mut S, event: Event<I>) -> Result<(), Error> {
         let serialized = postcard::to_allocvec(&event)?;
-        let flags: Flag = LLMP_FLAG_INITIALIZED;
+        let flags: Flags = LLMP_FLAG_INITIALIZED;
 
         match self.compressor.compress(&serialized)? {
             Some(comp_buf) => {
@@ -524,6 +525,9 @@ where
     }
 }
 
+/// Sets up a restarting fuzzer, using the [`StdShMemProvider`], and standard features.
+/// The restarting mgr is a combination of restarter and runner, that can be used on systems with and without `fork` support.
+/// The restarter will spawn a new process each time the child crashes or timeouts.
 #[cfg(feature = "std")]
 #[allow(clippy::type_complexity)]
 pub fn setup_restarting_mgr_std<I, S, ST>(
@@ -548,7 +552,7 @@ where
     setup_restarting_mgr(StdShMemProvider::new()?, stats, broker_port)
 }
 
-/// A restarting state is a combination of restarter and runner, that can be used on systems without `fork`.
+/// A restarting state is a combination of restarter and runner, that can be used on systems with and without `fork` support.
 /// The restarter will start a new process each time the child crashes or timeouts.
 #[cfg(feature = "std")]
 #[allow(
@@ -610,7 +614,9 @@ where
             #[cfg(windows)]
             let child_status = startable_self()?.status()?;
 
-            if unsafe { read_volatile(&(*receiver.current_recv_map.page()).size_used) } == 0 {
+            if unsafe { read_volatile(addr_of!((*receiver.current_recv_map.page()).size_used)) }
+                == 0
+            {
                 #[cfg(unix)]
                 if child_status == 137 {
                     // Out of Memory, see https://tldp.org/LDP/abs/html/exitcodes.html
