@@ -1,8 +1,9 @@
 /*!
-Welcome to libAFL
+Welcome to `LibAFL`
 */
 
 #![cfg_attr(not(feature = "std"), no_std)]
+#![cfg_attr(feature = "RUSTC_IS_NIGHTLY", feature(specialization))]
 
 #[macro_use]
 extern crate alloc;
@@ -172,11 +173,12 @@ mod tests {
     use crate::events::SimpleEventManager;
 
     #[test]
+    #[allow(clippy::similar_names)]
     fn test_fuzzer() {
         let rand = StdRand::with_seed(0);
 
         let mut corpus = InMemoryCorpus::<BytesInput>::new();
-        let testcase = Testcase::new(vec![0; 4]).into();
+        let testcase = Testcase::new(vec![0; 4]);
         corpus.add(testcase).unwrap();
 
         let mut state = State::new(
@@ -194,7 +196,6 @@ mod tests {
 
         let mut harness = |_buf: &[u8]| ExitKind::Ok;
         let mut executor = InProcessExecutor::new(
-            "main",
             &mut harness,
             tuple_list!(),
             //Box::new(|_, _, _, _, _| ()),
@@ -211,7 +212,7 @@ mod tests {
         for i in 0..1000 {
             fuzzer
                 .fuzz_one(&mut state, &mut executor, &mut event_manager, &scheduler)
-                .expect(&format!("Error in iter {}", i));
+                .unwrap_or_else(|_| panic!("Error in iter {}", i));
         }
 
         let state_serialized = postcard::to_allocvec(&state).unwrap();
