@@ -4,18 +4,20 @@ use alloc::{
     string::{String, ToString},
     vec::Vec,
 };
+use core::slice::from_raw_parts_mut;
 use serde::{Deserialize, Serialize};
 
 use crate::{
     bolts::{
-        ownedref::{OwnedArrayPtrMut, OwnedPtr},
+        ownedref::{OwnedRefMut, OwnedSliceMut},
         tuples::Named,
     },
+    executors::HasExecHooks,
     observers::Observer,
     Error,
 };
 
-/// A MapObserver observes the static map, as oftentimes used for afl-like coverage information
+/// A [`MapObserver`] observes the static map, as oftentimes used for afl-like coverage information
 pub trait MapObserver<T>: Observer
 where
     T: Default + Copy,
@@ -56,29 +58,35 @@ where
 /// The Map Observer retrieves the state of a map,
 /// that will get updated by the target.
 /// A well-known example is the AFL-Style coverage map.
-#[derive(Serialize, Deserialize, Clone, Debug)]
+#[derive(Serialize, Deserialize, Debug)]
 #[serde(bound = "T: serde::de::DeserializeOwned")]
 #[allow(clippy::unsafe_derive_deserialize)]
-pub struct StdMapObserver<T>
+pub struct StdMapObserver<'a, T>
 where
     T: Default + Copy + 'static + serde::Serialize + serde::de::DeserializeOwned,
 {
-    map: OwnedArrayPtrMut<T>,
+    map: OwnedSliceMut<'a, T>,
     initial: T,
     name: String,
 }
 
-impl<T> Observer for StdMapObserver<T>
+impl<'a, T> Observer for StdMapObserver<'a, T> where
+    T: Default + Copy + 'static + serde::Serialize + serde::de::DeserializeOwned
+{
+}
+
+impl<'a, EM, I, S, T> HasExecHooks<EM, I, S> for StdMapObserver<'a, T>
 where
     T: Default + Copy + 'static + serde::Serialize + serde::de::DeserializeOwned,
+    Self: MapObserver<T>,
 {
     #[inline]
-    fn pre_exec(&mut self) -> Result<(), Error> {
+    fn pre_exec(&mut self, _state: &mut S, _mgr: &mut EM, _input: &I) -> Result<(), Error> {
         self.reset_map()
     }
 }
 
-impl<T> Named for StdMapObserver<T>
+impl<'a, T> Named for StdMapObserver<'a, T>
 where
     T: Default + Copy + 'static + serde::Serialize + serde::de::DeserializeOwned,
 {
@@ -88,7 +96,7 @@ where
     }
 }
 
-impl<T> MapObserver<T> for StdMapObserver<T>
+impl<'a, T> MapObserver<T> for StdMapObserver<'a, T>
 where
     T: Default + Copy + 'static + serde::Serialize + serde::de::DeserializeOwned,
 {
@@ -118,38 +126,40 @@ where
     }
 }
 
-impl<T> StdMapObserver<T>
+impl<'a, T> StdMapObserver<'a, T>
 where
     T: Default + Copy + 'static + serde::Serialize + serde::de::DeserializeOwned,
 {
-    /// Creates a new MapObserver
-    pub fn new(name: &'static str, map: &'static mut [T], len: usize) -> Self {
-        assert!(map.len() >= len);
+    /// Creates a new [`MapObserver`]
+    #[must_use]
+    pub fn new(name: &'static str, map: &'a mut [T]) -> Self {
         let initial = if map.is_empty() { T::default() } else { map[0] };
         Self {
-            map: OwnedArrayPtrMut::ArrayPtr((map.as_mut_ptr(), len)),
+            map: OwnedSliceMut::Ref(map),
             name: name.to_string(),
             initial,
         }
     }
 
-    /// Creates a new MapObserver with an owned map
+    /// Creates a new [`MapObserver`] with an owned map
+    #[must_use]
     pub fn new_owned(name: &'static str, map: Vec<T>) -> Self {
         let initial = if map.is_empty() { T::default() } else { map[0] };
         Self {
-            map: OwnedArrayPtrMut::Owned(map),
+            map: OwnedSliceMut::Owned(map),
             name: name.to_string(),
             initial,
         }
     }
 
-    /// Creates a new MapObserver from a raw pointer
+    /// Creates a new [`MapObserver`] from a raw pointer
+    ///
     /// # Safety
-    /// Will dereference the map_ptr with up to len elements.
+    /// Will dereference the `map_ptr` with up to len elements.
     pub unsafe fn new_from_ptr(name: &'static str, map_ptr: *mut T, len: usize) -> Self {
         let initial = if len > 0 { *map_ptr } else { T::default() };
         StdMapObserver {
-            map: OwnedArrayPtrMut::ArrayPtr((map_ptr, len)),
+            map: OwnedSliceMut::Ref(from_raw_parts_mut(map_ptr, len)),
             name: name.to_string(),
             initial,
         }
@@ -157,30 +167,35 @@ where
 }
 
 /// Overlooking a variable bitmap
-#[derive(Serialize, Deserialize, Clone, Debug)]
+#[derive(Serialize, Deserialize, Debug)]
 #[serde(bound = "T: serde::de::DeserializeOwned")]
 #[allow(clippy::unsafe_derive_deserialize)]
-pub struct VariableMapObserver<T>
+pub struct VariableMapObserver<'a, T>
 where
     T: Default + Copy + 'static + serde::Serialize + serde::de::DeserializeOwned,
 {
-    map: OwnedArrayPtrMut<T>,
-    size: OwnedPtr<usize>,
+    map: OwnedSliceMut<'a, T>,
+    size: OwnedRefMut<'a, usize>,
     initial: T,
     name: String,
 }
 
-impl<T> Observer for VariableMapObserver<T>
+impl<'a, T> Observer for VariableMapObserver<'a, T> where
+    T: Default + Copy + 'static + serde::Serialize + serde::de::DeserializeOwned
+{
+}
+
+impl<'a, EM, I, S, T> HasExecHooks<EM, I, S> for VariableMapObserver<'a, T>
 where
     T: Default + Copy + 'static + serde::Serialize + serde::de::DeserializeOwned,
 {
     #[inline]
-    fn pre_exec(&mut self) -> Result<(), Error> {
+    fn pre_exec(&mut self, _state: &mut S, _mgr: &mut EM, _input: &I) -> Result<(), Error> {
         self.reset_map()
     }
 }
 
-impl<T> Named for VariableMapObserver<T>
+impl<'a, T> Named for VariableMapObserver<'a, T>
 where
     T: Default + Copy + 'static + serde::Serialize + serde::de::DeserializeOwned,
 {
@@ -190,7 +205,7 @@ where
     }
 }
 
-impl<T> MapObserver<T> for VariableMapObserver<T>
+impl<'a, T> MapObserver<T> for VariableMapObserver<'a, T>
 where
     T: Default + Copy + 'static + serde::Serialize + serde::de::DeserializeOwned,
 {
@@ -225,34 +240,35 @@ where
     }
 }
 
-impl<T> VariableMapObserver<T>
+impl<'a, T> VariableMapObserver<'a, T>
 where
     T: Default + Copy + 'static + serde::Serialize + serde::de::DeserializeOwned,
 {
-    /// Creates a new MapObserver
-    pub fn new(name: &'static str, map: &'static mut [T], size: *const usize) -> Self {
+    /// Creates a new [`MapObserver`]
+    pub fn new(name: &'static str, map: &'a mut [T], size: &'a mut usize) -> Self {
         let initial = if map.is_empty() { T::default() } else { map[0] };
         Self {
-            map: OwnedArrayPtrMut::ArrayPtr((map.as_mut_ptr(), map.len())),
-            size: OwnedPtr::Ptr(size),
+            map: OwnedSliceMut::Ref(map),
+            size: OwnedRefMut::Ref(size),
             name: name.into(),
             initial,
         }
     }
 
-    /// Creates a new MapObserver from a raw pointer
+    /// Creates a new [`MapObserver`] from a raw pointer
+    ///
     /// # Safety
-    /// Dereferences map_ptr with up to max_len elements of size_ptr.
+    /// Dereferences `map_ptr` with up to `max_len` elements of size.
     pub unsafe fn new_from_ptr(
         name: &'static str,
         map_ptr: *mut T,
         max_len: usize,
-        size_ptr: *const usize,
+        size: &'a mut usize,
     ) -> Self {
         let initial = if max_len > 0 { *map_ptr } else { T::default() };
         VariableMapObserver {
-            map: OwnedArrayPtrMut::ArrayPtr((map_ptr, max_len)),
-            size: OwnedPtr::Ptr(size_ptr),
+            map: OwnedSliceMut::Ref(from_raw_parts_mut(map_ptr, max_len)),
+            size: OwnedRefMut::Ref(size),
             name: name.into(),
             initial,
         }
@@ -264,7 +280,7 @@ where
 #[serde(bound = "M: serde::de::DeserializeOwned")]
 pub struct HitcountsMapObserver<M>
 where
-    M: MapObserver<u8>,
+    M: serde::Serialize + serde::de::DeserializeOwned,
 {
     base: M,
 }
@@ -284,27 +300,29 @@ static COUNT_CLASS_LOOKUP: [u8; 256] = [
     128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128,
 ];
 
-impl<M> Observer for HitcountsMapObserver<M>
+impl<M> Observer for HitcountsMapObserver<M> where M: MapObserver<u8> {}
+
+impl<EM, I, S, M> HasExecHooks<EM, I, S> for HitcountsMapObserver<M>
 where
-    M: MapObserver<u8>,
+    M: MapObserver<u8> + HasExecHooks<EM, I, S>,
 {
     #[inline]
-    fn pre_exec(&mut self) -> Result<(), Error> {
-        self.reset_map()
+    fn pre_exec(&mut self, state: &mut S, mgr: &mut EM, input: &I) -> Result<(), Error> {
+        self.base.pre_exec(state, mgr, input)
     }
 
     #[inline]
-    fn post_exec(&mut self) -> Result<(), Error> {
+    fn post_exec(&mut self, state: &mut S, mgr: &mut EM, input: &I) -> Result<(), Error> {
         for x in self.map_mut().iter_mut() {
             *x = COUNT_CLASS_LOOKUP[*x as usize];
         }
-        Ok(())
+        self.base.post_exec(state, mgr, input)
     }
 }
 
 impl<M> Named for HitcountsMapObserver<M>
 where
-    M: MapObserver<u8>,
+    M: Named + serde::Serialize + serde::de::DeserializeOwned,
 {
     #[inline]
     fn name(&self) -> &str {
@@ -349,9 +367,9 @@ where
 
 impl<M> HitcountsMapObserver<M>
 where
-    M: MapObserver<u8>,
+    M: serde::Serialize + serde::de::DeserializeOwned,
 {
-    /// Creates a new MapObserver
+    /// Creates a new [`MapObserver`]
     pub fn new(base: M) -> Self {
         Self { base }
     }
